@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -26,16 +26,26 @@
    Boston, MA 02111-1307, USA.
 */
 
-
+#include "config.h"
 #include "sis315_mmio.h"
+#include "direct/util.h"
 
 u32 sis_rl(volatile u8 *mmio, unsigned int offset)
 {
+#ifdef WORDS_BIGENDIAN
+     u32 r = *(volatile u32 *)(mmio + offset);
+     return BSWAP32(r);
+#else
 	return *(volatile u32 *)(mmio + offset);
+#endif
 }
 
 void sis_wl(volatile u8 *mmio, unsigned int offset, u32 value)
 {
+#ifdef WORDS_BIGENDIAN
+	*(volatile u32 *)(mmio + offset) = BSWAP32(value);
+#else
 	*(volatile u32 *)(mmio + offset) = value;
+#endif
 }
 
