@@ -27,6 +27,7 @@ struct DFBPixelFormatName {
      { DSPF_NV16, "NV16" }, \
      { DSPF_ARGB2554, "ARGB2554" }, \
      { DSPF_ARGB4444, "ARGB4444" }, \
+     { DSPF_RGBA4444, "RGBA4444" }, \
      { DSPF_NV21, "NV21" }, \
      { DSPF_AYUV, "AYUV" }, \
      { DSPF_A4, "A4" }, \
@@ -37,6 +38,11 @@ struct DFBPixelFormatName {
      { DSPF_RGB444, "RGB444" }, \
      { DSPF_RGB555, "RGB555" }, \
      { DSPF_BGR555, "BGR555" }, \
+     { DSPF_RGBA5551, "RGBA5551" }, \
+     { DSPF_YUV444P, "YUV444P" }, \
+     { DSPF_ARGB8565, "ARGB8565" }, \
+     { DSPF_AVYU, "AVYU" }, \
+     { DSPF_VYU, "VYU" }, \
      { DSPF_UNKNOWN, "UNKNOWN" } \
 };
 
@@ -90,10 +96,15 @@ struct DFBSurfaceBlittingFlagsName {
      { DSBLIT_SRC_PREMULTCOLOR, "SRC_PREMULTCOLOR" }, \
      { DSBLIT_XOR, "XOR" }, \
      { DSBLIT_INDEX_TRANSLATION, "INDEX_TRANSLATION" }, \
+     { DSBLIT_ROTATE90, "ROTATE90" }, \
      { DSBLIT_ROTATE180, "ROTATE180" }, \
+     { DSBLIT_ROTATE270, "ROTATE270" }, \
      { DSBLIT_COLORKEY_PROTECT, "COLORKEY_PROTECT" }, \
      { DSBLIT_SRC_MASK_ALPHA, "SRC_MASK_ALPHA" }, \
      { DSBLIT_SRC_MASK_COLOR, "SRC_MASK_COLOR" }, \
+     { DSBLIT_SOURCE2, "SOURCE2" }, \
+     { DSBLIT_FLIP_HORIZONTAL, "FLIP_HORIZONTAL" }, \
+     { DSBLIT_FLIP_VERTICAL, "FLIP_VERTICAL" }, \
      { DSBLIT_NOFX, "NOFX" } \
 };
 
@@ -202,6 +213,7 @@ struct DFBWindowCapabilitiesName {
      { DWCAPS_DOUBLEBUFFER, "DOUBLEBUFFER" }, \
      { DWCAPS_INPUTONLY, "INPUTONLY" }, \
      { DWCAPS_NODECORATION, "NODECORATION" }, \
+     { DWCAPS_SUBWINDOW, "SUBWINDOW" }, \
      { DWCAPS_COLOR, "COLOR" }, \
      { DWCAPS_NOFOCUS, "NOFOCUS" }, \
      { DWCAPS_NONE, "NONE" } \
@@ -227,6 +239,7 @@ struct DFBWindowOptionsName {
      { DWOP_SCALE, "SCALE" }, \
      { DWOP_KEEP_ABOVE, "KEEP_ABOVE" }, \
      { DWOP_KEEP_UNDER, "KEEP_UNDER" }, \
+     { DWOP_FOLLOW_BOUNDS, "FOLLOW_BOUNDS" }, \
      { DWOP_NONE, "NONE" } \
 };
 
@@ -298,6 +311,7 @@ struct DFBScreenEncoderTVStandardsName {
      { DSETV_PAL_N, "PAL_N" }, \
      { DSETV_PAL_NC, "PAL_NC" }, \
      { DSETV_NTSC_M_JPN, "NTSC_M_JPN" }, \
+     { DSETV_NTSC_443, "NTSC_443" }, \
      { DSETV_DIGITAL, "DIGITAL" }, \
      { DSETV_UNKNOWN, "UNKNOWN" } \
 };
@@ -449,6 +463,25 @@ struct DFBScreenEncoderScanModeName {
 };
 
 
+struct DFBScreenEncoderFrequencyName {
+     DFBScreenEncoderFrequency frequency;
+     const char *name;
+};
+
+#define DirectFBScreenEncoderFrequencyNames(Identifier) struct DFBScreenEncoderFrequencyName Identifier[] = { \
+     { DSEF_25HZ, "25HZ" }, \
+     { DSEF_29_97HZ, "29_97HZ" }, \
+     { DSEF_50HZ, "50HZ" }, \
+     { DSEF_59_94HZ, "59_94HZ" }, \
+     { DSEF_60HZ, "60HZ" }, \
+     { DSEF_75HZ, "75HZ" }, \
+     { DSEF_30HZ, "30HZ" }, \
+     { DSEF_24HZ, "24HZ" }, \
+     { DSEF_23_976HZ, "23_976HZ" }, \
+     { DSEF_UNKNOWN, "UNKNOWN" } \
+};
+
+
 struct DFBAccelerationMaskName {
      DFBAccelerationMask mask;
      const char *name;
@@ -462,6 +495,7 @@ struct DFBAccelerationMaskName {
      { DFXL_BLIT, "BLIT" }, \
      { DFXL_STRETCHBLIT, "STRETCHBLIT" }, \
      { DFXL_TEXTRIANGLES, "TEXTRIANGLES" }, \
+     { DFXL_BLIT2, "BLIT2" }, \
      { DFXL_DRAWSTRING, "DRAWSTRING" }, \
      { DFXL_NONE, "NONE" } \
 };
