@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -383,12 +383,6 @@ driver_open_device( CoreInputDevice  *device,
                D_PERROR( "DirectFB/Keyboard: Could not dup() file descriptor of TTY!\n" );
                return DFB_INIT;
           }
-
-          /* put keyboard into medium raw mode */
-          if (ioctl( fd, KDSKBMODE, K_MEDIUMRAW ) < 0) {
-               D_PERROR( "DirectFB/Keyboard: K_MEDIUMRAW failed!\n" );
-               return DFB_INIT;
-          }
      }
      else {
           fd = open( "/dev/tty0", O_RDWR | O_NOCTTY );
@@ -419,6 +413,11 @@ driver_open_device( CoreInputDevice  *device,
 
      tcsetpgrp( data->vt_fd, getpgrp() );
 
+     if (dfb_system_type() == CORE_FBDEV && dfb_config->vt) {
+          FBDev *dfb_fbdev = dfb_system_data();
+          dfb_fbdev->vt->flush = false;
+     }
+
      /* fill device info structure */
      snprintf( info->desc.name,
                DFB_INPUT_DEVICE_DESC_NAME_LENGTH, "Keyboard" );
@@ -517,13 +516,6 @@ driver_close_device( void *driver_data )
      if (tcsetattr( data->vt_fd, TCSAFLUSH, &data->old_ts ) < 0)
           D_PERROR("DirectFB/keyboard: tcsetattr for original values failed!\n");
 
-     if (dfb_system_type() == CORE_FBDEV && dfb_config->vt) {
-          if (ioctl( data->vt_fd, KDSKBMODE, K_XLATE ) < 0)
-               D_PERROR("DirectFB/keyboard: Could not set mode to XLATE!\n");
-          if (ioctl( data->vt_fd, KDSETMODE, KD_TEXT ) < 0)
-               D_PERROR("DirectFB/keyboard: Could not set terminal mode to text!\n");
-     }
-
      close( data->vt_fd );
 
      /* free private data */
