@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -384,7 +384,8 @@ IDirectFBVideoProvider_V4L_PlayTo( IDirectFBVideoProvider *thiz,
                 * Because we're constantly writing to the surface we
                 * permanently lock it.
                 */
-               ret = dfb_surface_lock_buffer( surface, CSBR_BACK, CSAF_GPU_WRITE, &data->destinationlock );
+               ret = dfb_surface_lock_buffer( surface, CSBR_BACK, CSAID_GPU,
+                                              CSAF_WRITE, &data->destinationlock );
 
                if (ret) {
                     pthread_mutex_unlock( &data->lock );
@@ -764,7 +765,8 @@ GrabThread( DirectThread *thread, void *ctx )
           h = surface->config.size.h;
           src = data->buffer + data->vmbuf.offsets[frame];
           
-          dfb_surface_lock_buffer( surface, CSBR_BACK, CSAF_CPU_WRITE, &data->destinationlock );
+          dfb_surface_lock_buffer( surface, CSBR_BACK, CSAID_CPU,
+                                   CSAF_WRITE, &data->destinationlock );
           dst       = data->destinationlock.addr;
           dst_pitch = data->destinationlock.pitch;
 
@@ -913,6 +915,7 @@ v4l_to_surface_overlay( CoreSurface *surface, DFBRectangle *rect,
                palette = VIDEO_PALETTE_RGB555;
                break;
           case DSPF_RGB16:
+          case DSPF_ARGB8565:
                bpp = 16;
                palette = VIDEO_PALETTE_RGB565;
                break;
@@ -1044,6 +1047,7 @@ v4l_to_surface_grab( CoreSurface *surface, DFBRectangle *rect,
                palette = VIDEO_PALETTE_RGB555;
                break;
           case DSPF_RGB16:
+          case DSPF_ARGB8565:
                palette = VIDEO_PALETTE_RGB565;
                break;
           case DSPF_RGB24:
@@ -1284,7 +1288,7 @@ V4L2_Thread( DirectThread *thread, void *ctx )
                h = surface->config.size.h;
                src = data->ptr[cur.index];
 
-               dfb_surface_lock_buffer( surface, CSBR_BACK, CSAF_CPU_WRITE, &lock );
+               dfb_surface_lock_buffer( surface, CSBR_BACK, CSAID_CPU, CSAF_WRITE, &lock );
                dst       = lock.addr;
                dst_pitch = lock.pitch;
                while (h--) {
@@ -1379,6 +1383,7 @@ v4l2_playto( CoreSurface *surface, DFBRectangle *rect, IDirectFBVideoProvider_V4
                palette = V4L2_PIX_FMT_RGB555;
                break;
           case DSPF_RGB16:
+          case DSPF_ARGB8565:
                palette = V4L2_PIX_FMT_RGB565;
                break;
           case DSPF_RGB24:
@@ -1393,7 +1398,7 @@ v4l2_playto( CoreSurface *surface, DFBRectangle *rect, IDirectFBVideoProvider_V4
                return DFB_UNSUPPORTED;
      }
 
-     err = dfb_surface_lock_buffer( surface, CSBR_BACK, CSAF_GPU_WRITE, &data->destinationlock );
+     err = dfb_surface_lock_buffer( surface, CSBR_BACK, CSAID_GPU, CSAF_WRITE, &data->destinationlock );
      if (err)
           return err;
 
