@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -167,6 +167,8 @@ direct_modules_unregister( DirectModuleDir *directory,
      D_MAGIC_ASSERT( entry, DirectModuleEntry );
 
      D_FREE( entry->name );
+     if (entry->file)
+          D_FREE( entry->file );
 
      direct_list_remove( &directory->entries, &entry->link );
 
@@ -182,7 +184,6 @@ int
 direct_modules_explore_directory( DirectModuleDir *directory )
 {
 #ifdef DYNAMIC_LINKING
-     int            dir_len;
      DIR           *dir;
      struct dirent *entry = NULL;
      struct dirent  tmp;
@@ -198,18 +199,16 @@ direct_modules_explore_directory( DirectModuleDir *directory )
 
      path = directory->path;
 
-     if(path[0]!='/') {
+     if (path[0] != '/') {
           pathfront = direct_config->module_dir;
-          if(!pathfront)
+          if (!pathfront)
                pathfront = MODULEDIR;
      }
 
      buf = alloca( strlen(pathfront) + 1 + strlen(path) + 1 ); /* pre, slash, post, 0 */
-     sprintf( buf, "%s%s%s", pathfront, ( pathfront && path && (path[0] != '/') && (pathfront[strlen(pathfront)-1] != '/') ) ? "/" : "", path );
-
-     dir_len = strlen( buf );
-     dir     = opendir( buf );
+     sprintf( buf, "%s/%s", pathfront, path );
 
+     dir = opendir( buf );
      if (!dir) {
           D_DEBUG_AT( Direct_Modules, "  -> ERROR opening directory: %s!\n", strerror(errno) );
           return 0;
@@ -238,7 +237,11 @@ direct_modules_explore_directory( DirectModuleDir *directory )
           module->directory = directory;
           module->dynamic   = true;
           module->file      = D_STRDUP( entry->d_name );
-
+          if (!module->file) {
+               D_MAGIC_CLEAR( module );
+               D_FREE( module );
+               continue;
+          }
 
           directory->loading = module;
 
@@ -426,24 +429,28 @@ static void *
 open_module( DirectModuleEntry *module )
 {
      DirectModuleDir *directory;
-     void            *handle;
-     char            *pathfront;
+     const char      *pathfront = "";
      const char      *path;
+     char            *buf;
+     void            *handle;
 
      D_MAGIC_ASSERT( module, DirectModuleEntry );
 
+     D_ASSERT( module->file != NULL );
+     D_ASSERT( module->directory != NULL );
+     D_ASSERT( module->directory->path != NULL );
+
      directory = module->directory;
      path      = directory->path;
-     pathfront = "";
 
-     if(path[0]!='/') {
+     if (path[0] != '/') {
           pathfront = direct_config->module_dir;
-          if(!pathfront)
+          if (!pathfront)
                pathfront = MODULEDIR;
      }
 
-     char buf[ strlen(pathfront) + 1 + strlen(path) + 1 + strlen(module->file) + 1 ];
-     sprintf( buf, "%s%s%s/%s", pathfront, ( pathfront && path && (path[0] != '/') && (pathfront[strlen(pathfront)-1] != '/') ) ? "/" : "", path, module->file );
+     buf = alloca( strlen( pathfront ) + 1 + strlen( path ) + 1 + strlen( module->file ) + 1 );
+     sprintf( buf, "%s/%s/%s", pathfront, path, module->file );
 
      D_DEBUG_AT( Direct_Modules, "Loading '%s'...\n", buf );
 
