@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -61,7 +62,7 @@
 #endif
 
 #define MAX_BUFFERS 200
-#define MAX_LEVEL   256
+#define MAX_LEVEL   200
 
 #define NAME_LEN    92
 
@@ -89,7 +90,11 @@ struct __D_DirectTraceBuffer {
 
 static DirectTraceBuffer *buffers[MAX_BUFFERS];
 static int                buffers_num  = 0;
-static pthread_mutex_t    buffers_lock = DIRECT_UTIL_RECURSIVE_PTHREAD_MUTEX_INITIALIZER;
+#ifdef HAVE_DECL_PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP
+static pthread_mutex_t    buffers_lock = PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP;
+#else
+static pthread_mutex_t    buffers_lock = PTHREAD_MUTEX_INITIALIZER;
+#endif
 static pthread_key_t      trace_key    = -1;
 
 /**************************************************************************************************/
@@ -122,7 +127,7 @@ buffer_destroy( void *arg )
 
 __attribute__((no_instrument_function))
 static inline DirectTraceBuffer *
-get_trace_buffer()
+get_trace_buffer( void )
 {
      DirectTraceBuffer *buffer;
 
@@ -156,8 +161,6 @@ get_trace_buffer()
 
 /**************************************************************************************************/
 
-#ifdef DYNAMIC_LINKING
-
 typedef struct {
      long offset;
      char name[NAME_LEN];
@@ -173,7 +176,11 @@ typedef struct {
 } SymbolTable;
 
 static DirectLink      *tables      = NULL;
-static pthread_mutex_t  tables_lock = DIRECT_UTIL_RECURSIVE_PTHREAD_MUTEX_INITIALIZER;
+#ifdef HAVE_DECL_PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP
+static pthread_mutex_t  tables_lock = PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP;
+#else
+static pthread_mutex_t  tables_lock = PTHREAD_MUTEX_INITIALIZER;
+#endif
 
 
 __attribute__((no_instrument_function))
@@ -207,9 +214,7 @@ add_symbol( SymbolTable *table, long offset, const char *name )
 
      symbol->offset = offset;
 
-     strncpy( symbol->name, name, NAME_LEN - 1 );
-
-     symbol->name[NAME_LEN - 1] = 0;
+     direct_snputs( symbol->name, name, NAME_LEN );
 }
 
 __attribute__((no_instrument_function))
@@ -217,16 +222,39 @@ static SymbolTable *
 load_symbols( const char *filename )
 {
      SymbolTable *table;
-     FILE        *pipe;
+     FILE        *fp = NULL;
+     bool         is_pipe = false;
      char         file[1024];
      char         line[1024];
      int          command_len;
      char        *command;
      const char  *full_path = filename;
+     char        *tmp;
 
-     if (access( filename, R_OK ) < 0 && errno == ENOENT) {
+     if (filename) {
+          if (access( filename, R_OK ) < 0 && errno == ENOENT) {
+               int len;
+
+               if ((len = readlink( "/proc/self/exe", file, sizeof(file) - 1 )) < 0) {
+                    D_PERROR( "Direct/Trace: readlink( \"/proc/self/exe\" ) failed!\n" );
+                    return NULL;
+               }
+
+               file[len] = 0;
+
+
+               tmp = strrchr( file, '/' );
+               if (!tmp)
+                    return NULL;
+
+               if (strcmp( filename, tmp + 1 ))
+                    return NULL;
+
+               full_path = file;
+          }
+     }
+     else {
           int   len;
-          char *tmp;
 
           if ((len = readlink( "/proc/self/exe", file, sizeof(file) - 1 )) < 0) {
                D_PERROR( "Direct/Trace: readlink( \"/proc/self/exe\" ) failed!\n" );
@@ -235,57 +263,73 @@ load_symbols( const char *filename )
 
           file[len] = 0;
 
-          tmp = strrchr( file, '/' ) + 1;
-          if (!tmp)
-               return NULL;
-
-          if (strcmp( filename, tmp ))
-               return NULL;
-
           full_path = file;
      }
 
      command_len = strlen( full_path ) + 32;
      command     = alloca( command_len );
 
-     snprintf( command, command_len, "nm -n %s", full_path );
-
-     pipe = popen( command, "r" );
-     if (!pipe) {
-          D_PERROR( "Direct/Trace: popen( \"%s\", \"r\" ) failed!\n", command );
+     /* First check if there's an "nm-n" file. */
+     tmp = strrchr( full_path, '/' );
+     if (!tmp)
           return NULL;
+
+     *tmp = 0;
+     snprintf( command, command_len, "%s/nm-n.%s", full_path, tmp + 1 );
+     *tmp = '/';
+
+     if (access( command, R_OK ) == 0) {
+          fp = fopen( command, "r" );
+          if (!fp)
+               D_PERROR( "Direct/Trace: fopen( \"%s\", \"r\" ) failed!\n", command );
+     }
+     else {
+          snprintf( command, command_len, "%s.nm", full_path );
+          if (access( command, R_OK ) == 0) {
+               fp = fopen( command, "r" );
+               if (!fp)
+                    D_PERROR( "Direct/Trace: fopen( \"%s\", \"r\" ) failed!\n", command );
+          }
+     }
+
+     /* Fallback to live mode. */
+     if (!fp) {
+          snprintf( command, command_len, "nm -n %s", full_path );
+
+          fp = popen( command, "r" );
+          if (!fp) {
+               D_PERROR( "Direct/Trace: popen( \"%s\", \"r\" ) failed!\n", command );
+               return NULL;
+          }
+
+          is_pipe = true;
      }
 
      table = calloc( 1, sizeof(SymbolTable) );
      if (!table) {
-          D_WARN( "out of memory" );
-          pclose( pipe );
-          return NULL;
+          D_OOM();
+          goto out;
      }
 
-     table->filename = strdup( filename );
-     if (!table->filename) {
-          D_WARN( "out of memory" );
-          free( table );
-          pclose( pipe );
-          return NULL;
-     }
+     if (filename)
+          table->filename = strdup( filename );
 
-     while (fgets( line, sizeof(line), pipe )) {
+     while (fgets( line, sizeof(line), fp )) {
           int  n;
+          int  digits = sizeof(long) * 2;
           long offset = 0;
           int  length = strlen(line);
 
-          if (length < 13 || line[length-1] != '\n')
+          if (line[0] == ' ' || length < (digits + 5) || line[length-1] != '\n')
                continue;
 
-          if (line[9] != 't' && line[9] != 'T' && line[9] != 'W')
+          if (line[digits + 1] != 't' && line[digits + 1] != 'T' && line[digits + 1] != 'W')
                continue;
 
-          if (line[8] != ' ' || line[10] != ' ' || line[11] == '.')
+          if (line[digits] != ' ' || line[digits + 2] != ' ' || line[digits + 3] == '.')
                continue;
 
-          for (n=0; n<8; n++) {
+          for (n=0; n<digits; n++) {
                char c = line[n];
 
                offset <<= 4;
@@ -298,10 +342,14 @@ load_symbols( const char *filename )
 
           line[length-1] = 0;
 
-          add_symbol( table, offset, line + 11 );
+          add_symbol( table, offset, line + digits + 3 );
      }
 
-     pclose( pipe );
+out:
+     if (is_pipe)
+          pclose( fp );
+     else
+          fclose( fp );
 
      return table;
 }
@@ -310,7 +358,7 @@ __attribute__((no_instrument_function))
 static int
 compare_symbols(const void *x, const void *y)
 {
-     return  *((long*) x)  -  *((long*) y);
+     return  *((const long*) x)  -  *((const long*) y);
 }
 
 __attribute__((no_instrument_function))
@@ -319,17 +367,27 @@ find_table( const char *filename )
 {
      SymbolTable *table;
 
-     direct_list_foreach (table, tables) {
-          if (!strcmp( filename, table->filename ))
-               return table;
+     if (filename) {
+          direct_list_foreach (table, tables) {
+               if (table->filename && !strcmp( filename, table->filename ))
+                    return table;
+          }
+     }
+     else {
+          direct_list_foreach (table, tables) {
+               if (!table->filename)
+                    return table;
+          }
      }
 
      return NULL;
 }
 
+/**************************************************************************************************/
+
 __attribute__((no_instrument_function))
-static const char *
-lookup_symbol( const char *filename, long offset )
+const char *
+direct_trace_lookup_symbol( const char *filename, long offset )
 {
      Symbol      *symbol;
      SymbolTable *table;
@@ -355,18 +413,106 @@ lookup_symbol( const char *filename, long offset )
      return symbol ? symbol->name : NULL;
 }
 
+__attribute__((no_instrument_function))
+const char *
+direct_trace_lookup_file( void *address, void **ret_base )
+{
+#ifdef DYNAMIC_LINKING
+     Dl_info info;
+
+     if (dladdr( address, &info )) {
+          if (ret_base)
+               *ret_base = info.dli_fbase;
+
+          return info.dli_fname;
+     }
+     else
+#endif
+     {
+          if (ret_base)
+               *ret_base = NULL;
+     }
+
+     return NULL;
+}
+
+__attribute__((no_instrument_function))
+static void
+direct_trace_print_stack_recursive( DirectTraceBuffer *buffer, int level, int levels )
+{
+     void *fn = buffer->trace[level].addr;
+
+#ifdef DYNAMIC_LINKING
+     Dl_info info;
 #endif
 
-/**************************************************************************************************/
+     if (level == levels) {
+          /* lock, print header, unroll stack */
+
+          direct_log_lock( NULL );
+
+          if (buffer->name)
+               direct_log_printf( NULL, "(-) [%5d: -STACK- '%s']\n", buffer->tid, buffer->name );
+          else
+               direct_log_printf( NULL, "(-) [%5d: -STACK- ]\n", buffer->tid );
+
+          return;
+     }
+
+#ifdef DYNAMIC_LINKING
+     if (dladdr( fn, &info )) {
+          if (info.dli_fname) {
+               const char *symbol = NULL;//info.dli_sname;
+
+               if (!symbol) {
+                    symbol = direct_trace_lookup_symbol(info.dli_fname, (long)(fn - info.dli_fbase));
+                    if (!symbol) {
+                         symbol = direct_trace_lookup_symbol(info.dli_fname, (long)(fn));
+                         if (!symbol) {
+                              if (info.dli_sname)
+                                   symbol = info.dli_sname;
+                              else
+                                   symbol = "??";
+                         }
+                    }
+               }
+
+               direct_trace_print_stack_recursive( buffer, level+1, levels );
+               direct_log_printf( NULL, "  #%-2d 0x%08lx in %s () from %s [%p]\n",
+                                  levels - level - 1, (unsigned long) fn, symbol, info.dli_fname, info.dli_fbase );
+          }
+          else if (info.dli_sname) {
+               direct_trace_print_stack_recursive( buffer, level+1, levels );
+               direct_log_printf( NULL, "  #%-2d 0x%08lx in %s ()\n",
+                                  levels - level - 1, (unsigned long) fn, info.dli_sname );
+          }
+          else {
+               direct_trace_print_stack_recursive( buffer, level+1, levels );
+               direct_log_printf( NULL, "  #%-2d 0x%08lx in ?? ()\n",
+                                  levels - level - 1, (unsigned long) fn );
+          }
+     }
+     else
+#endif
+     {
+          const char *symbol = direct_trace_lookup_symbol(NULL, (long)(fn));
+
+          direct_trace_print_stack_recursive( buffer, level+1, levels );
+          direct_log_printf( NULL, "  #%-2d 0x%08lx in %s ()\n",
+                             levels - level - 1, (unsigned long) fn, symbol ? symbol : "??" );
+     }
+
+     if (level == 0) {
+          /* stack unrolled, print end, unlock */
+          direct_log_printf( NULL, "\n" );
+          direct_log_unlock( NULL );
+     }
+}
 
 __attribute__((no_instrument_function))
 void
 direct_trace_print_stack( DirectTraceBuffer *buffer )
 {
-#ifdef DYNAMIC_LINKING
-     Dl_info info;
-#endif
-     int     i;
      int     level;
 
      if (!direct_config->trace)
@@ -391,56 +537,15 @@ direct_trace_print_stack( DirectTraceBuffer *buffer )
           return;
      }
 
-
-     if (buffer->name)
-          direct_log_printf( NULL, "(-) [%5d: -STACK- '%s']\n", buffer->tid, buffer->name );
-     else
-          direct_log_printf( NULL, "(-) [%5d: -STACK- ]\n", buffer->tid );
-
-     for (i=level-1; i>=0; i--) {
-          void *fn = buffer->trace[i].addr;
-
-          direct_log_printf( NULL, "  #%-2d 0x%08lx in ", level - i - 1, (unsigned long) fn );
-
-#ifdef DYNAMIC_LINKING
-          if (dladdr( fn, &info )) {
-               if (info.dli_fname) {
-                    const char *symbol = NULL;//info.dli_sname;
-
-                    if (!symbol) {
-                         symbol = lookup_symbol(info.dli_fname, (long)(fn - info.dli_fbase));
-                         if (!symbol) {
-                              symbol = lookup_symbol(info.dli_fname, (long)(fn));
-                              if (!symbol) {
-                                   if (info.dli_sname)
-                                        symbol = info.dli_sname;
-                                   else
-                                        symbol = "??";
-                              }
-                         }
-                    }
-
-                    direct_log_printf( NULL, "%s () from %s\n", symbol, info.dli_fname );
-               }
-               else if (info.dli_sname) {
-                    direct_log_printf( NULL, "%s ()\n", info.dli_sname );
-               }
-               else
-                    direct_log_printf( NULL, "?? ()\n" );
-          }
-          else
-#endif
-               direct_log_printf( NULL, "?? ()\n" );
-     }
-
-     direct_log_printf( NULL, "\n" );
+     /* we are starting with the last entry (0) */
+     direct_trace_print_stack_recursive( buffer, 0, level );
 
      buffer->in_trace = false;
 }
 
 __attribute__((no_instrument_function))
 void
-direct_trace_print_stacks()
+direct_trace_print_stacks( void )
 {
      int                i;
      DirectTraceBuffer *buffer = get_trace_buffer();
@@ -460,7 +565,7 @@ direct_trace_print_stacks()
 
 __attribute__((no_instrument_function))
 int
-direct_trace_debug_indent()
+direct_trace_debug_indent( void )
 {
      int                in;
      DirectTraceBuffer *buffer = get_trace_buffer();
@@ -489,7 +594,13 @@ direct_trace_copy_buffer( DirectTraceBuffer *buffer )
      if (!buffer)
           buffer = get_trace_buffer();
 
-     copy = calloc( 1, sizeof(DirectTraceBuffer) );
+     level = buffer->level;
+     if (level > MAX_LEVEL) {
+          D_WARN( "only copying %d of %d items", MAX_LEVEL, level );
+          level = MAX_LEVEL;
+     }
+
+     copy = calloc( 1, sizeof(*buffer) - sizeof(buffer->trace) + sizeof(buffer->trace[0]) * level );
      if (!copy)
           return NULL;
 
@@ -499,12 +610,6 @@ direct_trace_copy_buffer( DirectTraceBuffer *buffer )
      copy->tid   = buffer->tid;
      copy->level = buffer->level;
 
-     level = buffer->level;
-     if (level > MAX_LEVEL) {
-          D_WARN( "only copying %d of %d items", MAX_LEVEL, level );
-          level = MAX_LEVEL;
-     }
-
      direct_memcpy( copy->trace, buffer->trace, level * sizeof(buffer->trace[0]) );
 
      return copy;
@@ -552,22 +657,35 @@ __cyg_profile_func_exit (void *this_fn,
      }
 }
 
-
-
 #else
 
+const char *
+direct_trace_lookup_symbol( const char *filename, long offset )
+{
+     return NULL;
+}
+
+const char *
+direct_trace_lookup_file( void *address, void **ret_base )
+{
+     if (ret_base)
+          *ret_base = NULL;
+          
+     return NULL;
+}
+
 void
 direct_trace_print_stack( DirectTraceBuffer *buffer )
 {
 }
 
 void
-direct_trace_print_stacks()
+direct_trace_print_stacks( void )
 {
 }
 
 int
-direct_trace_debug_indent()
+direct_trace_debug_indent( void )
 {
      return 0;
 }
