@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -67,7 +67,7 @@ IDirectFBDisplayLayer_Dispatcher_Destruct( IDirectFBDisplayLayer *thiz )
 
 /**************************************************************************************************/
 
-static DFBResult
+static DirectResult
 IDirectFBDisplayLayer_Dispatcher_AddRef( IDirectFBDisplayLayer *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer_Dispatcher)
@@ -77,7 +77,7 @@ IDirectFBDisplayLayer_Dispatcher_AddRef( IDirectFBDisplayLayer *thiz )
      return DFB_OK;
 }
 
-static DFBResult
+static DirectResult
 IDirectFBDisplayLayer_Dispatcher_Release( IDirectFBDisplayLayer *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer_Dispatcher)
@@ -493,6 +493,29 @@ Dispatch_GetConfiguration( IDirectFBDisplayLayer *thiz, IDirectFBDisplayLayer *r
                                     VMBT_NONE );
 }
 
+static DirectResult
+Dispatch_TestConfiguration( IDirectFBDisplayLayer *thiz, IDirectFBDisplayLayer *real,
+                            VoodooManager *manager, VoodooRequestMessage *msg )
+{
+     DirectResult                 ret;
+     const DFBDisplayLayerConfig *config;
+     VoodooMessageParser          parser;
+     DFBDisplayLayerConfigFlags   failed;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer_Dispatcher)
+
+     VOODOO_PARSER_BEGIN( parser, msg );
+     VOODOO_PARSER_GET_DATA( parser, config );
+     VOODOO_PARSER_END( parser );
+
+     ret = real->TestConfiguration( real, config, &failed );
+
+     return voodoo_manager_respond( manager, msg->header.serial,
+                                    ret, VOODOO_INSTANCE_NONE,
+                                    VMBT_UINT, failed,
+                                    VMBT_NONE );
+}
+
 static DirectResult
 Dispatch_SetCooperativeLevel( IDirectFBDisplayLayer *thiz, IDirectFBDisplayLayer *real,
                               VoodooManager *manager, VoodooRequestMessage *msg )
@@ -567,6 +590,9 @@ Dispatch( void *dispatcher, void *real, VoodooManager *manager, VoodooRequestMes
           case IDIRECTFBDISPLAYLAYER_METHOD_ID_GetConfiguration:
                return Dispatch_GetConfiguration( dispatcher, real, manager, msg );
 
+          case IDIRECTFBDISPLAYLAYER_METHOD_ID_TestConfiguration:
+               return Dispatch_TestConfiguration( dispatcher, real, manager, msg );
+
           case IDIRECTFBDISPLAYLAYER_METHOD_ID_CreateWindow:
                return Dispatch_CreateWindow( dispatcher, real, manager, msg );
      }
