@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -97,7 +97,7 @@ IDirectFBSurface_Requestor_Destruct( IDirectFBSurface *thiz )
 
 /**************************************************************************************************/
 
-static DFBResult
+static DirectResult
 IDirectFBSurface_Requestor_AddRef( IDirectFBSurface *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Requestor)
@@ -107,7 +107,7 @@ IDirectFBSurface_Requestor_AddRef( IDirectFBSurface *thiz )
      return DFB_OK;
 }
 
-static DFBResult
+static DirectResult
 IDirectFBSurface_Requestor_Release( IDirectFBSurface *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Requestor)
@@ -1101,6 +1101,66 @@ IDirectFBSurface_Requestor_SetIndexTranslation( IDirectFBSurface *thiz,
      return DFB_UNIMPLEMENTED;
 }
 
+static DFBResult
+IDirectFBSurface_Requestor_Write( IDirectFBSurface   *thiz,
+                                  const DFBRectangle *rect,
+                                  const void         *ptr,
+                                  int                 pitch )
+{
+     DFBResult    ret = DFB_OK;
+     int          y;
+     DFBRectangle r;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Requestor)
+
+     if (!rect || !ptr || pitch < 1)
+          return DFB_INVARG;
+
+     r.x = rect->x;
+     r.y = rect->y;
+     r.w = rect->w;
+     r.h = 1;
+
+     for (y=0; y<rect->h; y++) {
+          ret = voodoo_manager_request( data->manager, data->instance,
+                                        IDIRECTFBSURFACE_METHOD_ID_Write, VREQ_QUEUE, NULL,
+                                        VMBT_DATA, sizeof(DFBRectangle), &r,
+                                        VMBT_DATA, pitch /* FIXME */, ptr + y * pitch,
+                                        VMBT_INT, pitch,
+                                        VMBT_NONE );
+          if (ret)
+               break;
+
+          r.y++;
+     }
+
+     return ret;
+}
+
+static DFBResult
+IDirectFBSurface_Requestor_SetRenderOptions( IDirectFBSurface        *thiz,
+                                             DFBSurfaceRenderOptions  options )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Requestor)
+
+     return voodoo_manager_request( data->manager, data->instance,
+                                    IDIRECTFBSURFACE_METHOD_ID_SetRenderOptions, VREQ_QUEUE, NULL,
+                                    VMBT_INT, options,
+                                    VMBT_NONE );
+}
+
+static DFBResult
+IDirectFBSurface_Requestor_SetMatrix( IDirectFBSurface *thiz,
+                                      const s32        *matrix )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Requestor)
+
+     return voodoo_manager_request( data->manager, data->instance,
+                                    IDIRECTFBSURFACE_METHOD_ID_SetMatrix, VREQ_QUEUE, NULL,
+                                    VMBT_DATA, sizeof(s32) * 9, matrix,
+                                    VMBT_NONE );
+}
+
 
 /**************************************************************************************************/
 
@@ -1189,6 +1249,11 @@ Construct( IDirectFBSurface *thiz,
      
      thiz->SetIndexTranslation = IDirectFBSurface_Requestor_SetIndexTranslation;
 
+     thiz->Write = IDirectFBSurface_Requestor_Write;
+
+     thiz->SetRenderOptions = IDirectFBSurface_Requestor_SetRenderOptions;
+     thiz->SetMatrix = IDirectFBSurface_Requestor_SetMatrix;
+
      return DFB_OK;
 }
 
