@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -250,6 +250,9 @@ typedef struct _GraphicsDeviceFuncs {
      bool (*Blit)            ( void *driver_data, void *device_data,
                                DFBRectangle *rect, int dx, int dy );
 
+     bool (*Blit2)           ( void *driver_data, void *device_data,
+                               DFBRectangle *rect, int dx, int dy, int sx2, int sy2 );
+
      bool (*StretchBlit)     ( void *driver_data, void *device_data,
                                DFBRectangle *srect, DFBRectangle *drect );
 
@@ -354,6 +357,12 @@ void dfb_gfxcard_batchblit              ( DFBRectangle         *rects,
                                           int                   num,
                                           CardState            *state );
 
+void dfb_gfxcard_batchblit2             ( DFBRectangle         *rects,
+                                          DFBPoint             *points,
+                                          DFBPoint             *points2,
+                                          int                   num,
+                                          CardState            *state );
+
 void dfb_gfxcard_tileblit               ( DFBRectangle         *rect,
                                           int                   dx1,
                                           int                   dy1,
@@ -376,15 +385,17 @@ void dfb_gfxcard_drawstring             ( const u8             *text,
                                           int                   x,
                                           int                   y,
                                           CoreFont             *font,
+                                          unsigned int          layers, 
                                           CardState            *state );
 
-void dfb_gfxcard_drawglyph              ( unsigned int          index,
+void dfb_gfxcard_drawglyph              ( CoreGlyphData       **glyph,
                                           int                   x,
                                           int                   y,
                                           CoreFont             *font,
+                                          unsigned int          layers, 
                                           CardState            *state );
 
-void dfb_gfxcard_drawstring_check_state ( CoreFont             *font,
+bool dfb_gfxcard_drawstring_check_state ( CoreFont             *font,
                                           CardState            *state );
 
 DFBResult dfb_gfxcard_sync( void );
