@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -429,23 +429,23 @@ dfb_wm_close_all_stacks( void *data )
           D_DEBUG_AT( Core_WM, "  -> checking %p...\n", stack );
 
           D_MAGIC_ASSERT( stack, CoreWindowStack );
-          
+
           context = stack->context;
-          D_ASSERT( context != NULL );
-          
+          D_MAGIC_ASSERT( context, CoreLayerContext );
+
           D_DEBUG_AT( Core_WM, "  -> ref context %p...\n", context );
 
           dfb_layer_context_ref( context );
-          
+
           dfb_layer_context_lock( context );
-          
+
           if (stack->flags & CWSF_INITIALIZED) {
                D_DEBUG_AT( Core_WM, "  => CLOSING %p\n", stack );
                dfb_wm_close_stack( stack );
           }
 
           dfb_layer_context_unlock( context );
-          
+
           D_DEBUG_AT( Core_WM, "  -> unref context %p...\n", context );
 
           dfb_layer_context_unref( context );
@@ -536,6 +536,9 @@ dfb_wm_init_stack( CoreWindowStack *stack )
      D_MAGIC_ASSERT( stack, CoreWindowStack );
      D_ASSERT( !(stack->flags & CWSF_INITIALIZED) );
 
+     D_MAGIC_ASSERT( stack->context, CoreLayerContext );
+     FUSION_SKIRMISH_ASSERT( &stack->context->lock );
+
      /* Allocate shared stack data. */
      if (wm_shared->info.stack_data_size) {
           if (stack->stack_data)
@@ -585,7 +588,7 @@ dfb_wm_close_stack( CoreWindowStack *stack )
           return DFB_OK;
      }
 
-     D_ASSERT( stack->context != NULL );
+     D_MAGIC_ASSERT( stack->context, CoreLayerContext );
      FUSION_SKIRMISH_ASSERT( &stack->context->lock );
 
      /* Deactivate before deinitialization. */
@@ -620,6 +623,9 @@ dfb_wm_set_active( CoreWindowStack *stack,
      D_MAGIC_ASSERT( stack, CoreWindowStack );
      D_ASSERT( stack->flags & CWSF_INITIALIZED );
 
+     D_MAGIC_ASSERT( stack->context, CoreLayerContext );
+     FUSION_SKIRMISH_ASSERT( &stack->context->lock );
+
      if (active) {
           D_ASSUME( !(stack->flags & CWSF_ACTIVATED) );
 
@@ -658,6 +664,9 @@ dfb_wm_resize_stack( CoreWindowStack *stack,
      D_MAGIC_ASSERT( stack, CoreWindowStack );
      D_ASSERT( stack->flags & CWSF_INITIALIZED );
 
+     D_MAGIC_ASSERT( stack->context, CoreLayerContext );
+     FUSION_SKIRMISH_ASSERT( &stack->context->lock );
+
      /* Notify window manager about the new size. */
      return wm_local->funcs->ResizeStack( stack, wm_local->data, stack->stack_data, width, height );
 }
@@ -675,6 +684,9 @@ dfb_wm_process_input( CoreWindowStack     *stack,
      D_MAGIC_ASSERT( stack, CoreWindowStack );
      D_ASSERT( stack->flags & CWSF_INITIALIZED );
 
+     D_MAGIC_ASSERT( stack->context, CoreLayerContext );
+     FUSION_SKIRMISH_ASSERT( &stack->context->lock );
+
      D_ASSERT( event != NULL );
 
      /* Dispatch input event via window manager. */
@@ -693,6 +705,9 @@ dfb_wm_flush_keys( CoreWindowStack *stack )
      D_MAGIC_ASSERT( stack, CoreWindowStack );
      D_ASSERT( stack->flags & CWSF_INITIALIZED );
 
+     D_MAGIC_ASSERT( stack->context, CoreLayerContext );
+     FUSION_SKIRMISH_ASSERT( &stack->context->lock );
+
      return wm_local->funcs->FlushKeys( stack, wm_local->data, stack->stack_data );
 }
 
@@ -711,6 +726,9 @@ dfb_wm_window_at( CoreWindowStack  *stack,
      D_MAGIC_ASSERT( stack, CoreWindowStack );
      D_ASSERT( stack->flags & CWSF_INITIALIZED );
 
+     D_MAGIC_ASSERT( stack->context, CoreLayerContext );
+     FUSION_SKIRMISH_ASSERT( &stack->context->lock );
+
      D_ASSERT( ret_window != NULL );
 
      return wm_local->funcs->WindowAt( stack, wm_local->data, stack->stack_data, x, y, ret_window );
@@ -730,6 +748,9 @@ dfb_wm_window_lookup( CoreWindowStack  *stack,
      D_MAGIC_ASSERT( stack, CoreWindowStack );
      D_ASSERT( stack->flags & CWSF_INITIALIZED );
 
+     D_MAGIC_ASSERT( stack->context, CoreLayerContext );
+     FUSION_SKIRMISH_ASSERT( &stack->context->lock );
+
      D_ASSERT( ret_window != NULL );
 
      return wm_local->funcs->WindowLookup( stack, wm_local->data,
@@ -750,6 +771,9 @@ dfb_wm_enum_windows( CoreWindowStack      *stack,
      D_MAGIC_ASSERT( stack, CoreWindowStack );
      D_ASSERT( stack->flags & CWSF_INITIALIZED );
 
+     D_MAGIC_ASSERT( stack->context, CoreLayerContext );
+     FUSION_SKIRMISH_ASSERT( &stack->context->lock );
+
      D_ASSERT( callback != NULL );
 
      return wm_local->funcs->EnumWindows( stack, wm_local->data,
@@ -771,6 +795,9 @@ dfb_wm_get_insets( CoreWindowStack *stack,
      D_MAGIC_ASSERT( stack, CoreWindowStack );
      D_ASSERT( stack->flags & CWSF_INITIALIZED );
 
+     D_MAGIC_ASSERT( stack->context, CoreLayerContext );
+     FUSION_SKIRMISH_ASSERT( &stack->context->lock );
+
      D_ASSERT( window != NULL );
      D_ASSERT( insets != NULL );
 
@@ -795,6 +822,9 @@ dfb_wm_preconfigure_window( CoreWindowStack *stack,
      D_ASSERT( window != NULL );
      D_ASSERT( wm_local->funcs->PreConfigureWindow != NULL );
 
+     D_MAGIC_ASSERT( stack->context, CoreLayerContext );
+     FUSION_SKIRMISH_ASSERT( &stack->context->lock );
+
      D_DEBUG_AT( Core_WM, "%s( %p, %p [%d,%d-%dx%d] )\n", __FUNCTION__,
                  stack, window, DFB_RECTANGLE_VALS(&window->config.bounds) );
 
@@ -839,6 +869,9 @@ dfb_wm_add_window( CoreWindowStack *stack,
      D_MAGIC_ASSERT( stack, CoreWindowStack );
      D_ASSERT( stack->flags & CWSF_INITIALIZED );
 
+     D_MAGIC_ASSERT( stack->context, CoreLayerContext );
+     FUSION_SKIRMISH_ASSERT( &stack->context->lock );
+
      D_ASSERT( window != NULL );
 
      D_DEBUG_AT( Core_WM, "%s( %p, %p [%d,%d-%dx%d] )\n", __FUNCTION__,
@@ -868,6 +901,9 @@ dfb_wm_remove_window( CoreWindowStack *stack,
      D_MAGIC_ASSERT( stack, CoreWindowStack );
      D_ASSERT( stack->flags & CWSF_INITIALIZED );
 
+     D_MAGIC_ASSERT( stack->context, CoreLayerContext );
+     FUSION_SKIRMISH_ASSERT( &stack->context->lock );
+
      D_ASSERT( window != NULL );
 
      D_DEBUG_AT( Core_WM, "%s( %p, %p [%d,%d-%dx%d] )\n", __FUNCTION__,
@@ -901,6 +937,9 @@ dfb_wm_set_window_property( CoreWindowStack  *stack,
      D_MAGIC_ASSERT( stack, CoreWindowStack );
      D_ASSERT( stack->flags & CWSF_INITIALIZED );
 
+     D_MAGIC_ASSERT( stack->context, CoreLayerContext );
+     FUSION_SKIRMISH_ASSERT( &stack->context->lock );
+
      D_ASSERT( window != NULL );
      D_ASSERT( key != NULL );
 
@@ -928,6 +967,9 @@ dfb_wm_get_window_property( CoreWindowStack  *stack,
      D_MAGIC_ASSERT( stack, CoreWindowStack );
      D_ASSERT( stack->flags & CWSF_INITIALIZED );
 
+     D_MAGIC_ASSERT( stack->context, CoreLayerContext );
+     FUSION_SKIRMISH_ASSERT( &stack->context->lock );
+
      D_ASSERT( window != NULL );
      D_ASSERT( key != NULL );
 
@@ -954,6 +996,9 @@ dfb_wm_remove_window_property( CoreWindowStack  *stack,
      D_MAGIC_ASSERT( stack, CoreWindowStack );
      D_ASSERT( stack->flags & CWSF_INITIALIZED );
 
+     D_MAGIC_ASSERT( stack->context, CoreLayerContext );
+     FUSION_SKIRMISH_ASSERT( &stack->context->lock );
+
      D_ASSERT( window != NULL );
      D_ASSERT( key != NULL );
 
@@ -976,6 +1021,10 @@ dfb_wm_set_window_config( CoreWindow             *window,
      D_ASSERT( window != NULL );
      D_ASSERT( config != NULL );
 
+     D_MAGIC_ASSERT( window->stack, CoreWindowStack );
+     D_MAGIC_ASSERT( window->stack->context, CoreLayerContext );
+     FUSION_SKIRMISH_ASSERT( &window->stack->context->lock );
+
      D_DEBUG_AT( Core_WM, "%s( %p [%d,%d-%dx%d], %p, 0x%x )\n", __FUNCTION__,
                  window, DFB_RECTANGLE_VALS(&window->config.bounds), config, flags );
 
@@ -994,6 +1043,10 @@ dfb_wm_restack_window( CoreWindow *window,
 
      D_ASSERT( window != NULL );
 
+     D_MAGIC_ASSERT( window->stack, CoreWindowStack );
+     D_MAGIC_ASSERT( window->stack->context, CoreLayerContext );
+     FUSION_SKIRMISH_ASSERT( &window->stack->context->lock );
+
      D_ASSERT( relative == NULL || relative == window || relation != 0);
 
      D_DEBUG_AT( Core_WM, "%s( %p [%d,%d-%dx%d], %p, %d )\n", __FUNCTION__,
@@ -1013,6 +1066,10 @@ dfb_wm_grab( CoreWindow *window,
 
      D_ASSERT( window != NULL );
 
+     D_MAGIC_ASSERT( window->stack, CoreWindowStack );
+     D_MAGIC_ASSERT( window->stack->context, CoreLayerContext );
+     FUSION_SKIRMISH_ASSERT( &window->stack->context->lock );
+
      D_ASSERT( grab != NULL );
 
      D_DEBUG_AT( Core_WM, "%s( %p [%d,%d-%dx%d], %d )\n", __FUNCTION__,
@@ -1031,6 +1088,10 @@ dfb_wm_ungrab( CoreWindow *window,
 
      D_ASSERT( window != NULL );
 
+     D_MAGIC_ASSERT( window->stack, CoreWindowStack );
+     D_MAGIC_ASSERT( window->stack->context, CoreLayerContext );
+     FUSION_SKIRMISH_ASSERT( &window->stack->context->lock );
+
      D_ASSERT( grab != NULL );
 
      D_DEBUG_AT( Core_WM, "%s( %p [%d,%d-%dx%d], %d )\n", __FUNCTION__,
@@ -1048,12 +1109,36 @@ dfb_wm_request_focus( CoreWindow *window )
 
      D_ASSERT( window != NULL );
 
+     D_MAGIC_ASSERT( window->stack, CoreWindowStack );
+     D_MAGIC_ASSERT( window->stack->context, CoreLayerContext );
+     FUSION_SKIRMISH_ASSERT( &window->stack->context->lock );
+
      D_DEBUG_AT( Core_WM, "%s( %p [%d,%d-%dx%d] )\n", __FUNCTION__,
                  window, DFB_RECTANGLE_VALS(&window->config.bounds) );
 
      return wm_local->funcs->RequestFocus( window, wm_local->data, window->window_data );
 }
 
+DFBResult
+dfb_wm_begin_updates( CoreWindow      *window,
+                      const DFBRegion *update )
+{
+     D_ASSERT( wm_local != NULL );
+     D_ASSERT( wm_local->funcs != NULL );
+     D_ASSERT( wm_local->funcs->RequestFocus != NULL );
+
+     D_ASSERT( window != NULL );
+
+     D_MAGIC_ASSERT( window->stack, CoreWindowStack );
+     D_MAGIC_ASSERT( window->stack->context, CoreLayerContext );
+     FUSION_SKIRMISH_ASSERT( &window->stack->context->lock );
+
+     D_DEBUG_AT( Core_WM, "%s( %p [%d,%d-%dx%d] )\n", __FUNCTION__,
+                 window, DFB_RECTANGLE_VALS(&window->config.bounds) );
+
+     return wm_local->funcs->BeginUpdates( window, wm_local->data, window->window_data, update );
+}
+
 DFBResult
 dfb_wm_update_stack( CoreWindowStack     *stack,
                      const DFBRegion     *region,
@@ -1066,6 +1151,9 @@ dfb_wm_update_stack( CoreWindowStack     *stack,
      D_MAGIC_ASSERT( stack, CoreWindowStack );
      D_ASSERT( stack->flags & CWSF_INITIALIZED );
 
+     D_MAGIC_ASSERT( stack->context, CoreLayerContext );
+     FUSION_SKIRMISH_ASSERT( &stack->context->lock );
+
      DFB_REGION_ASSERT( region );
 
      D_DEBUG_AT( Core_WM, "%s( %p, [%d,%d-%dx%d], 0x%x )\n", __FUNCTION__,
@@ -1086,6 +1174,10 @@ dfb_wm_update_window( CoreWindow          *window,
 
      D_ASSERT( window != NULL );
 
+     D_MAGIC_ASSERT( window->stack, CoreWindowStack );
+     D_MAGIC_ASSERT( window->stack->context, CoreLayerContext );
+     FUSION_SKIRMISH_ASSERT( &window->stack->context->lock );
+
      DFB_REGION_ASSERT_IF( region );
 
      D_DEBUG_AT( Core_WM, "%s( %p [%d,%d-%dx%d], [%d,%d-%dx%d], 0x%x )\n", __FUNCTION__,
@@ -1107,6 +1199,9 @@ dfb_wm_update_cursor( CoreWindowStack       *stack,
      D_MAGIC_ASSERT( stack, CoreWindowStack );
      D_ASSERT( stack->flags & CWSF_INITIALIZED );
 
+     D_MAGIC_ASSERT( stack->context, CoreLayerContext );
+     FUSION_SKIRMISH_ASSERT( &stack->context->lock );
+
      D_FLAGS_ASSERT( flags, CCUF_ALL );
 
      if (dfb_config->no_cursor_updates)
