@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -115,22 +115,12 @@ IDirectFBSurface_Destruct( IDirectFBSurface *thiz )
      dfb_state_set_destination( &data->state, NULL );
      dfb_state_set_source( &data->state, NULL );
      dfb_state_set_source_mask( &data->state, NULL );
+     dfb_state_set_source2( &data->state, NULL );
 
      dfb_state_destroy( &data->state );
 
-     if (data->font) {
-          IDirectFBFont      *font      = data->font;
-          IDirectFBFont_data *font_data = font->priv;
-
-          if (font_data) {
-               if (data->surface)
-                    dfb_font_drop_destination( font_data->font, data->surface );
-
-               font->Release( font );
-          }
-          else
-               D_WARN( "font dead?" );
-     }
+     if (data->font)
+          data->font->Release( data->font );
 
      if (data->surface) {
           if (data->locked)
@@ -195,8 +185,10 @@ IDirectFBSurface_GetPixelFormat( IDirectFBSurface      *thiz,
 static DFBResult
 IDirectFBSurface_GetAccelerationMask( IDirectFBSurface    *thiz,
                                       IDirectFBSurface    *source,
-                                      DFBAccelerationMask *mask )
+                                      DFBAccelerationMask *ret_mask )
 {
+     DFBAccelerationMask mask = DFXL_NONE;
+
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
      D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
@@ -204,34 +196,63 @@ IDirectFBSurface_GetAccelerationMask( IDirectFBSurface    *thiz,
      if (!data->surface)
           return DFB_DESTROYED;
 
-     if (!mask)
+     if (!ret_mask)
           return DFB_INVARG;
 
-     dfb_gfxcard_state_check( &data->state, DFXL_FILLRECTANGLE );
-     dfb_gfxcard_state_check( &data->state, DFXL_DRAWRECTANGLE );
-     dfb_gfxcard_state_check( &data->state, DFXL_DRAWLINE );
-     dfb_gfxcard_state_check( &data->state, DFXL_FILLTRIANGLE );
+     dfb_state_lock( &data->state );
 
+     /* Check drawing functions */
+     if (dfb_gfxcard_state_check( &data->state, DFXL_FILLRECTANGLE ))
+          mask |= DFXL_FILLRECTANGLE;
+
+     if (dfb_gfxcard_state_check( &data->state, DFXL_DRAWRECTANGLE ))
+          mask |= DFXL_DRAWRECTANGLE;
+
+     if (dfb_gfxcard_state_check( &data->state, DFXL_DRAWLINE ))
+          mask |= DFXL_DRAWLINE;
+
+     if (dfb_gfxcard_state_check( &data->state, DFXL_FILLTRIANGLE ))
+          mask |= DFXL_FILLTRIANGLE;
+
+     dfb_state_unlock( &data->state );
+
+     /* Check blitting functions */
      if (source) {
           IDirectFBSurface_data *src_data = source->priv;
 
           dfb_state_set_source( &data->state, src_data->surface );
+          dfb_state_set_source2( &data->state, data->surface ); // FIXME
 
-          dfb_gfxcard_state_check( &data->state, DFXL_BLIT );
-          dfb_gfxcard_state_check( &data->state, DFXL_STRETCHBLIT );
-          dfb_gfxcard_state_check( &data->state, DFXL_TEXTRIANGLES );
+          dfb_state_lock( &data->state );
+
+          if (dfb_gfxcard_state_check( &data->state, DFXL_BLIT ))
+               mask |= DFXL_BLIT;
+
+          if (dfb_gfxcard_state_check( &data->state, DFXL_BLIT2 ))
+               mask |= DFXL_BLIT2;
+
+          if (dfb_gfxcard_state_check( &data->state, DFXL_STRETCHBLIT ))
+               mask |= DFXL_STRETCHBLIT;
+
+          if (dfb_gfxcard_state_check( &data->state, DFXL_TEXTRIANGLES ))
+               mask |= DFXL_TEXTRIANGLES;
+
+          dfb_state_unlock( &data->state );
      }
 
+     /* Check text rendering function */
      if (data->font) {
           IDirectFBFont_data *font_data = data->font->priv;
 
-          dfb_gfxcard_drawstring_check_state( font_data->font, &data->state );
+          if (dfb_gfxcard_drawstring_check_state( font_data->font, &data->state ))
+               mask |= DFXL_DRAWSTRING;
      }
 
-     *mask = data->state.accel;
+     *ret_mask = mask;
 
      return DFB_OK;
 }
+
 static DFBResult
 IDirectFBSurface_GetPosition( IDirectFBSurface *thiz,
                           int              *x,
@@ -420,14 +441,14 @@ IDirectFBSurface_Lock( IDirectFBSurface *thiz,
           return DFB_INVAREA;
 
      if (flags & DSLF_READ)
-          access |= CSAF_CPU_READ;
+          access |= CSAF_READ;
 
      if (flags & DSLF_WRITE) {
-          access |= CSAF_CPU_WRITE;
+          access |= CSAF_WRITE;
           role = CSBR_BACK;
      }
 
-     ret = dfb_surface_lock_buffer( data->surface, role, access, &data->lock );
+     ret = dfb_surface_lock_buffer( data->surface, role, CSAID_CPU, access, &data->lock );
      if (ret)
           return ret;
 
@@ -790,6 +811,46 @@ IDirectFBSurface_SetColor( IDirectFBSurface *thiz,
           dfb_state_set_color_index( &data->state,
                                      dfb_palette_search( surface->palette, r, g, b, a ) );
 
+     data->state.colors[0]        = data->state.color;
+     data->state.color_indices[0] = data->state.color_index;
+
+     return DFB_OK;
+}
+
+static DFBResult
+IDirectFBSurface_SetColors( IDirectFBSurface *thiz,
+                            const DFBColorID *ids,
+                            const DFBColor   *colors,
+                            unsigned int      num )
+{
+     unsigned int  i;
+     CoreSurface  *surface;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
+
+     D_DEBUG_AT( Surface, "%s( %p, %p, %p, %u )\n", __FUNCTION__, thiz, ids, colors, num );
+
+     surface = data->surface;
+     if (!surface)
+          return DFB_DESTROYED;
+
+     for (i=0; i<num; i++) {
+          D_DEBUG_AT( Surface, "  -> [%d] id %d = %02x %02x %02x %02x\n",
+                      i, ids[i], colors[i].a, colors[i].r, colors[i].g, colors[i].b );
+
+          if (ids[i] >= DFB_COLOR_IDS_MAX)
+               return DFB_INVARG;
+
+          data->state.colors[ids[i]] = colors[i];
+
+          if (DFB_PIXELFORMAT_IS_INDEXED( surface->config.format ))
+               data->state.color_indices[ids[i]] = dfb_palette_search( surface->palette,
+                                                                       colors[i].r, colors[i].g, colors[i].b, colors[i].a );
+     }
+
+     dfb_state_set_color( &data->state, &data->state.colors[0] );
+     dfb_state_set_color_index( &data->state, data->state.color_indices[0] );
+
      return DFB_OK;
 }
 
@@ -822,6 +883,9 @@ IDirectFBSurface_SetColorIndex( IDirectFBSurface *thiz,
 
      dfb_state_set_color_index( &data->state, index );
 
+     data->state.colors[0]        = data->state.color;
+     data->state.color_indices[0] = data->state.color_index;
+
      return DFB_OK;
 }
 
@@ -1150,16 +1214,8 @@ IDirectFBSurface_SetFont( IDirectFBSurface *thiz,
               data->encoding = font_data->encoding;
          }
 
-         if (data->font) {
-              IDirectFBFont_data *old_data;
-              IDirectFBFont      *old = data->font;
-
-              DIRECT_INTERFACE_GET_DATA_FROM( old, old_data, IDirectFBFont );
-
-              dfb_font_drop_destination( old_data->font, data->surface );
-
-              old->Release( old );
-         }
+         if (data->font)
+              data->font->Release( data->font );
 
          data->font = font;
      }
@@ -1825,6 +1881,118 @@ IDirectFBSurface_BatchBlit( IDirectFBSurface   *thiz,
      return DFB_OK;
 }
 
+static DFBResult
+IDirectFBSurface_BatchBlit2( IDirectFBSurface   *thiz,
+                             IDirectFBSurface   *source,
+                             IDirectFBSurface   *source2,
+                             const DFBRectangle *source_rects,
+                             const DFBPoint     *dest_points,
+                             const DFBPoint     *source2_points,
+                             int                 num )
+{
+     int                    i, dx, dy, sx, sy, sx2, sy2;
+     DFBRectangle          *rects;
+     DFBPoint              *points;
+     DFBPoint              *points2;
+     IDirectFBSurface_data *src_data;
+     IDirectFBSurface_data *src2_data;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
+
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
+     if (!data->surface)
+          return DFB_DESTROYED;
+
+     if (!source || !source_rects || !dest_points || num < 1)
+          return DFB_INVARG;
+
+     if (!data->area.current.w || !data->area.current.h)
+          return DFB_INVAREA;
+
+     if (data->locked)
+          return DFB_LOCKED;
+
+
+     src_data = (IDirectFBSurface_data*)source->priv;
+
+     if (!src_data->area.current.w || !src_data->area.current.h)
+          return DFB_INVAREA;
+
+     src2_data = (IDirectFBSurface_data*)source2->priv;
+
+     if (!src2_data->area.current.w || !src2_data->area.current.h)
+          return DFB_INVAREA;
+
+
+     dx = data->area.wanted.x;
+     dy = data->area.wanted.y;
+
+     sx = src_data->area.wanted.x;
+     sy = src_data->area.wanted.y;
+
+     sx2 = src2_data->area.wanted.x;
+     sy2 = src2_data->area.wanted.y;
+
+     rects   = alloca( sizeof(DFBRectangle) * num );
+     points  = alloca( sizeof(DFBPoint) * num );
+     points2 = alloca( sizeof(DFBPoint) * num );
+
+     direct_memcpy( rects, source_rects, sizeof(DFBRectangle) * num );
+     direct_memcpy( points, dest_points, sizeof(DFBPoint) * num );
+     direct_memcpy( points2, source2_points, sizeof(DFBPoint) * num );
+
+     for (i=0; i<num; i++) {
+          DFBRectangle rect2;
+
+          rects[i].x += sx;
+          rects[i].y += sy;
+
+          points[i].x += dx;
+          points[i].y += dy;
+
+          points2[i].x += sx2;
+          points2[i].y += sy2;
+
+          if (!dfb_rectangle_intersect( &rects[i], &src_data->area.current ))
+               rects[i].w = rects[i].h = 0;
+          else {
+               points[i].x += rects[i].x - (source_rects[i].x + sx);
+               points[i].y += rects[i].y - (source_rects[i].y + sy);
+               points2[i].x += rects[i].x - (source_rects[i].x + sx);
+               points2[i].y += rects[i].y - (source_rects[i].y + sy);
+     
+     
+               rect2.x = points2[i].x;
+               rect2.y = points2[i].y;
+               rect2.w = rects[i].w;
+               rect2.h = rects[i].h;
+     
+               if (!dfb_rectangle_intersect( &rect2, &src2_data->area.current ))
+                    rects[i].w = rects[i].h = 0;
+
+               points[i].x += rect2.x - points2[i].x;
+               points[i].y += rect2.y - points2[i].y;
+               points2[i].x += rect2.x - points2[i].x;
+               points2[i].y += rect2.y - points2[i].y;
+
+               rects[i].w = rect2.w;
+               rects[i].h = rect2.h;
+          }
+     }
+
+     dfb_state_set_source( &data->state, src_data->surface );
+     dfb_state_set_source2( &data->state, src2_data->surface );
+
+     /* fetch the source color key from the source if necessary */
+     if (data->state.blittingflags & DSBLIT_SRC_COLORKEY)
+          dfb_state_set_src_colorkey( &data->state, src_data->src_key.value );
+
+     dfb_gfxcard_batchblit2( rects, points, points2, num, &data->state );
+
+     return DFB_OK;
+}
+
 static DFBResult
 IDirectFBSurface_StretchBlit( IDirectFBSurface   *thiz,
                               IDirectFBSurface   *source,
@@ -2045,6 +2213,7 @@ IDirectFBSurface_DrawString( IDirectFBSurface *thiz,
      IDirectFBFont      *font;
      IDirectFBFont_data *font_data;
      CoreFont           *core_font;
+     unsigned int        layers = 1;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
@@ -2077,6 +2246,13 @@ IDirectFBSurface_DrawString( IDirectFBSurface *thiz,
 
      core_font = font_data->font;
 
+     if (flags & DSTF_OUTLINE) {
+          if (!(core_font->attributes & DFFA_OUTLINED))
+               return DFB_UNSUPPORTED;
+
+          layers = 2;
+     }
+
      if (!(flags & DSTF_TOP)) {
           y -= core_font->ascender;
 
@@ -2105,7 +2281,7 @@ IDirectFBSurface_DrawString( IDirectFBSurface *thiz,
                unsigned int   current = indices[i];
                CoreGlyphData *glyph;
 
-               if (dfb_font_get_glyph_data( core_font, current, &glyph ) == DFB_OK) {
+               if (dfb_font_get_glyph_data( core_font, current, 0, &glyph ) == DFB_OK) {
                     width += glyph->advance;
 
                     if (prev && core_font->GetKerning &&
@@ -2127,7 +2303,7 @@ IDirectFBSurface_DrawString( IDirectFBSurface *thiz,
 
      dfb_gfxcard_drawstring( (const unsigned char*) text, bytes, data->encoding,
                              data->area.wanted.x + x, data->area.wanted.y + y,
-                             core_font, &data->state );
+                             core_font, layers, &data->state );
 
      return DFB_OK;
 }
@@ -2138,10 +2314,13 @@ IDirectFBSurface_DrawGlyph( IDirectFBSurface *thiz,
                             DFBSurfaceTextFlags flags )
 {
      DFBResult           ret;
+     int                 l;
      IDirectFBFont      *font;
      IDirectFBFont_data *font_data;
      CoreFont           *core_font;
+     CoreGlyphData      *glyph[DFB_FONT_MAX_LAYERS];
      unsigned int        index;
+     unsigned int        layers = 1;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
@@ -2169,7 +2348,13 @@ IDirectFBSurface_DrawGlyph( IDirectFBSurface *thiz,
 
      core_font = font_data->font;
 
-     /* FIXME: Avoid double locking. */
+     if (flags & DSTF_OUTLINE) {
+          if (!(core_font->attributes & DFFA_OUTLINED))
+               return DFB_UNSUPPORTED;
+
+          layers = 2;
+     }
+
      dfb_font_lock( core_font );
 
      ret = dfb_font_decode_character( core_font, data->encoding, character, &index );
@@ -2178,6 +2363,14 @@ IDirectFBSurface_DrawGlyph( IDirectFBSurface *thiz,
           return ret;
      }
 
+     for (l=0; l<layers; l++) {
+          ret = dfb_font_get_glyph_data( core_font, index, l, &glyph[l] );
+          if (ret) {
+               dfb_font_unlock( core_font );
+               return ret;
+          }
+     }
+
      if (!(flags & DSTF_TOP)) {
           y -= core_font->ascender;
 
@@ -2186,23 +2379,15 @@ IDirectFBSurface_DrawGlyph( IDirectFBSurface *thiz,
      }
 
      if (flags & (DSTF_RIGHT | DSTF_CENTER)) {
-          CoreGlyphData *glyph;
-
-          ret = dfb_font_get_glyph_data( core_font, index, &glyph );
-          if (ret) {
-               dfb_font_unlock( core_font );
-               return ret;
-          }
-
           if (flags & DSTF_RIGHT)
-               x -= glyph->advance;
+               x -= glyph[0]->advance;
           else if (flags & DSTF_CENTER)
-               x -= glyph->advance >> 1;
+               x -= glyph[0]->advance >> 1;
      }
 
-     dfb_gfxcard_drawglyph( index,
+     dfb_gfxcard_drawglyph( glyph,
                             data->area.wanted.x + x, data->area.wanted.y + y,
-                            core_font, &data->state );
+                            core_font, layers, &data->state );
 
      dfb_font_unlock( core_font );
 
@@ -2458,6 +2643,7 @@ IDirectFBSurface_ReleaseSource( IDirectFBSurface *thiz )
 
      dfb_state_set_source( &data->state, NULL );
      dfb_state_set_source_mask( &data->state, NULL );
+     dfb_state_set_source2( &data->state, NULL );
 
      return DFB_OK;
 }
@@ -2646,6 +2832,7 @@ DFBResult IDirectFBSurface_Construct( IDirectFBSurface       *thiz,
      thiz->Blit = IDirectFBSurface_Blit;
      thiz->TileBlit = IDirectFBSurface_TileBlit;
      thiz->BatchBlit = IDirectFBSurface_BatchBlit;
+     thiz->BatchBlit2 = IDirectFBSurface_BatchBlit2;
      thiz->StretchBlit = IDirectFBSurface_StretchBlit;
      thiz->TextureTriangles = IDirectFBSurface_TextureTriangles;
 
@@ -2682,6 +2869,8 @@ DFBResult IDirectFBSurface_Construct( IDirectFBSurface       *thiz,
      thiz->Write = IDirectFBSurface_Write;
      thiz->Read  = IDirectFBSurface_Read;
 
+     thiz->SetColors = IDirectFBSurface_SetColors;
+
      dfb_surface_attach( surface,
                          IDirectFBSurface_listener, thiz, &data->reaction );
 
