@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -1219,6 +1219,15 @@ IDirectFBWindow_SetDstGeometry( IDirectFBWindow         *thiz,
      return dfb_window_set_config( data->window, &config, CWCF_DST_GEOMETRY );
 }
 
+static DFBResult
+IDirectFBWindow_SetRotation( IDirectFBWindow *thiz,
+                             int              rotation )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
+
+     return dfb_window_set_rotation( data->window, rotation % 360 );     
+}
+
 static DFBResult
 IDirectFBWindow_SetAssociation( IDirectFBWindow *thiz,
                                 DFBWindowID      window_id )
@@ -1276,6 +1285,26 @@ IDirectFBWindow_GetApplicationID( IDirectFBWindow *thiz,
      return DFB_OK;
 }
 
+static DFBResult
+IDirectFBWindow_BeginUpdates( IDirectFBWindow *thiz,
+                              const DFBRegion *update )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow)
+
+     D_DEBUG_AT( IDirectFB_Window, "%s()\n", __FUNCTION__ );
+
+     if (data->destroyed)
+          return DFB_DESTROYED;
+
+     dfb_windowstack_lock( data->window->stack );
+
+     dfb_wm_begin_updates( data->window, update );
+
+     dfb_windowstack_unlock( data->window->stack );
+
+     return DFB_OK;
+}
+
 DFBResult
 IDirectFBWindow_Construct( IDirectFBWindow *thiz,
                            CoreWindow      *window,
@@ -1345,9 +1374,11 @@ IDirectFBWindow_Construct( IDirectFBWindow *thiz,
      thiz->UngrabUnselectedKeys = IDirectFBWindow_UngrabUnselectedKeys;
      thiz->SetSrcGeometry = IDirectFBWindow_SetSrcGeometry;
      thiz->SetDstGeometry = IDirectFBWindow_SetDstGeometry;
+     thiz->SetRotation = IDirectFBWindow_SetRotation;
      thiz->SetAssociation = IDirectFBWindow_SetAssociation;
      thiz->SetApplicationID = IDirectFBWindow_SetApplicationID;
      thiz->GetApplicationID = IDirectFBWindow_GetApplicationID;
+     thiz->BeginUpdates = IDirectFBWindow_BeginUpdates;
 
      return DFB_OK;
 }
