@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -113,11 +113,13 @@ primaryGetScreenSize( CoreScreen *screen,
      return DFB_OK;
 }
 
-ScreenFuncs sdlPrimaryScreenFuncs = {
+static ScreenFuncs primaryScreenFuncs = {
      .InitScreen    = primaryInitScreen,
      .GetScreenSize = primaryGetScreenSize,
 };
 
+ScreenFuncs *sdlPrimaryScreenFuncs = &primaryScreenFuncs;
+
 /******************************************************************************/
 
 static void * ScreenUpdateLoop( DirectThread *thread, void *arg );
@@ -309,7 +311,7 @@ primaryUpdateRegion( CoreLayer             *layer,
      return dfb_sdl_update_screen( dfb_sdl_core, NULL );
 }
 
-DisplayLayerFuncs sdlPrimaryLayerFuncs = {
+static DisplayLayerFuncs primaryLayerFuncs = {
      .LayerDataSize     = primaryLayerDataSize,
      .RegionDataSize    = primaryRegionDataSize,
      .InitLayer         = primaryInitLayer,
@@ -322,6 +324,8 @@ DisplayLayerFuncs sdlPrimaryLayerFuncs = {
      .UpdateRegion      = primaryUpdateRegion,
 };
 
+DisplayLayerFuncs *sdlPrimaryLayerFuncs = &primaryLayerFuncs;
+
 /******************************************************************************/
 
 static DFBResult
