@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -141,11 +141,13 @@ primaryGetScreenSize( CoreScreen *screen,
      return DFB_OK;
 }
 
-ScreenFuncs vncPrimaryScreenFuncs = {
+static ScreenFuncs primaryScreenFuncs = {
      .InitScreen    = primaryInitScreen,
      .GetScreenSize = primaryGetScreenSize,
 };
 
+ScreenFuncs *vncPrimaryScreenFuncs = &primaryScreenFuncs;
+
 /******************************************************************************/
 
 static int
@@ -385,7 +387,7 @@ primaryReallocateSurface( CoreLayer             *layer,
      return DFB_OK;
 }
 
-DisplayLayerFuncs vncPrimaryLayerFuncs = {
+static DisplayLayerFuncs primaryLayerFuncs = {
      .LayerDataSize     = primaryLayerDataSize,
      .RegionDataSize    = primaryRegionDataSize,
      .InitLayer         = primaryInitLayer,
@@ -401,6 +403,9 @@ DisplayLayerFuncs vncPrimaryLayerFuncs = {
      .ReallocateSurface = primaryReallocateSurface,
 };
 
+DisplayLayerFuncs *vncPrimaryLayerFuncs = &primaryLayerFuncs;
+
+
 /******************************************************************************/
 
 static DFBResult
@@ -416,7 +421,7 @@ update_screen( CoreSurface *surface, int x, int y, int w, int h )
      D_ASSERT( rfb_screen != NULL );
      D_ASSERT( rfb_screen->frameBuffer != NULL );
 
-     ret = dfb_surface_lock_buffer( surface, CSBR_FRONT, CSAF_CPU_READ, &lock );
+     ret = dfb_surface_lock_buffer( surface, CSBR_FRONT, CSAID_CPU, CSAF_READ, &lock );
      if (ret) {
           D_DERROR( ret, "DirectFB/VNC: Couldn't lock layer surface!\n" );
           return ret;
