@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -32,8 +32,15 @@
 #include <core/layers.h>
 #include <core/screens.h>
 
-extern ScreenFuncs       x11PrimaryScreenFuncs;
-extern DisplayLayerFuncs x11PrimaryLayerFuncs;
+#include "xwindow.h"
+
+extern ScreenFuncs       *x11PrimaryScreenFuncs;
+extern DisplayLayerFuncs *x11PrimaryLayerFuncs;
+
+typedef struct {
+     int      layer_id;
+     XWindow *xw;
+} X11LayerData;
 
 #endif // __X11__PRIMARY_H__
 
