@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -59,6 +59,7 @@
 
 #include <core/input_driver.h>
 
+D_DEBUG_DOMAIN( X11_Input, "X11/Input", "X11 Input/Key/Mouse handling" );
 
 DFB_INPUT_DRIVER( x11input )
 
@@ -276,7 +277,7 @@ xsymbol_to_id( KeySym xKeySymbol )
           case 0                   : break;
 
           default:
-               D_DEBUG("X11: Unknown key symbol 0x%lx\n", xKeySymbol);
+               D_DEBUG_AT( X11_Input, "Unknown key symbol 0x%lx\n", xKeySymbol);
      }    
 
      return DIKI_UNKNOWN;
@@ -453,9 +454,26 @@ static void handleMouseEvent(XEvent* pXEvent, X11InputData* pData)
 static void
 handle_expose( const XExposeEvent *expose )
 {
-     CoreLayer               *layer = dfb_layer_at( DLID_PRIMARY );
-     const DisplayLayerFuncs *funcs = layer->funcs;
+     CoreLayer               *layer = 0;
+     const DisplayLayerFuncs *funcs = 0;
      CoreLayerContext        *context;
+     int                      i;
+
+     /* find the correct layer */
+     for( i=0; i<dfb_layer_num(); i++ ) {
+          X11LayerData *lds;
+
+          layer = dfb_layer_at( i );
+          lds   = (X11LayerData*)(layer->layer_data);
+          if( lds->xw && (lds->xw->window == expose->window) )
+               break;
+     }
+
+     /* layer not found? */
+     if( i==dfb_layer_num() )
+          return;
+
+     funcs = layer->funcs;
 
      D_ASSERT( funcs != NULL );
      D_ASSERT( funcs->UpdateRegion != NULL );
@@ -508,10 +526,11 @@ x11EventThread( DirectThread *thread, void *driver_data )
           unsigned int  pull = 23;
           XEvent        xEvent; 
           DFBInputEvent dfbEvent;
+          static int    nextKeyIsRepeat = false;
 
           /* FIXME: Detect key repeats, we're receiving KeyPress, KeyRelease, KeyPress, KeyRelease... !!?? */
 
-          if (!shared->xw || !shared->xw->window) {
+          if (shared->window_count == 0) {
                /* no window, so no event */
                usleep( 50000 );
                continue;
@@ -524,8 +543,21 @@ x11EventThread( DirectThread *thread, void *driver_data )
           while (!data->stop && pull-- && XPending( x11->display )) {
                XNextEvent( x11->display, &xEvent );
 
+               /* is this key repeat? idea from GII */
+               if ( (xEvent.type == KeyRelease) && (XPending( x11->display )) ) {
+                    XEvent peekEvent;
+                    XPeekEvent( x11->display, &peekEvent );
+                    if ( (peekEvent.type == KeyPress) &&
+                         (peekEvent.xkey.keycode == xEvent.xkey.keycode) &&
+                         (peekEvent.xkey.time - xEvent.xkey.time < 2) ) {
+                              nextKeyIsRepeat = true;
+                    }
+               }
+
                XUnlockDisplay( x11->display );
 
+               D_DEBUG_AT( X11_Input, "Event received: %d\n", xEvent.type );
+
                switch (xEvent.type) {
                     case ButtonPress:
                     case ButtonRelease:
@@ -545,6 +577,11 @@ x11EventThread( DirectThread *thread, void *driver_data )
                          dfbEvent.timestamp.tv_sec  =  xEvent.xkey.time / 1000;
                          dfbEvent.timestamp.tv_usec = (xEvent.xkey.time % 1000) * 1000;
 
+                         if ( (xEvent.type == KeyPress) && nextKeyIsRepeat ) {
+                              nextKeyIsRepeat = false;
+                              dfbEvent.flags |= DIEF_REPEAT;
+                         }
+
                          dfb_input_dispatch( data->device, &dfbEvent );
                          break;
                     }
@@ -615,6 +652,8 @@ driver_open_device( CoreInputDevice  *device,
      DFBX11       *x11    = dfb_system_data();
      DFBX11Shared *shared = x11->shared;
 
+     D_DEBUG_AT( X11_Input, "%s()\n", __FUNCTION__ );
+
      fusion_skirmish_prevail( &shared->lock );
 
      fusion_skirmish_dismiss( &shared->lock );
@@ -699,20 +738,13 @@ driver_close_device( void *driver_data )
      DFBX11       *x11    = data->x11;
      DFBX11Shared *shared = x11->shared;
 
+     D_DEBUG_AT( X11_Input, "%s()\n", __FUNCTION__ );
+
      /* stop input thread */
      data->stop = true;
 
      XLockDisplay( x11->display );
 
-     if (shared->xw) {
-          XWindow *xw = shared->xw;
-
-          shared->xw = NULL;
-
-          /* the window must generate an event, otherwise the input thread will not end */
-          dfb_x11_close_window( x11, xw );
-     }
-
      XSync( x11->display, False );
 
      XUnlockDisplay( x11->display );
