@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -38,6 +38,7 @@
 
 #include "x11.h"
 
+D_DEBUG_DOMAIN( X11_Window, "X11/Window", "X11 Window handling" );
 
 static bool use_shm = true;
 
@@ -61,9 +62,11 @@ error_handler( Display *display, XErrorEvent *event )
 {
      char buf[512];
 
+     D_DEBUG_AT( X11_Window, "%s()\n", __FUNCTION__ );
+
      XGetErrorText( display, event->error_code, buf, sizeof(buf) );
 
-     D_ERROR( "X11/Display: Error! %s\n", buf );
+     D_ERROR( "X11/Window: Error! %s\n", buf );
 
      error_code = event->error_code;
 
@@ -76,7 +79,7 @@ dfb_x11_open_window( DFBX11 *x11, XWindow** ppXW, int iXPos, int iYPos, int iWid
      XWindow              *xw;
      XSetWindowAttributes  attr = { .background_pixmap = 0 };
 
-     D_INFO( "X11/Window: Creating %4dx%4d %s window...\n", iWidth, iHeight, dfb_pixelformat_name(format) );
+     D_DEBUG_AT( X11_Window, "Creating %4dx%4d %s window...\n", iWidth, iHeight, dfb_pixelformat_name(format) );
 
      xw = D_CALLOC( 1, sizeof(XWindow) );
      if (!xw)
@@ -89,10 +92,10 @@ dfb_x11_open_window( DFBX11 *x11, XWindow** ppXW, int iXPos, int iYPos, int iWid
 
      xw->screenptr = DefaultScreenOfDisplay(xw->display);
      xw->screennum = DefaultScreen(xw->display);
-     xw->depth     = DFB_COLOR_BITS_PER_PIXEL(format) + DFB_ALPHA_BITS_PER_PIXEL(format);
-     xw->visual    = x11->visuals[DFB_PIXELFORMAT_INDEX(format)] ?: DefaultVisualOfScreen(xw->screenptr);
+     xw->depth     = DefaultDepthOfScreen(xw->screenptr);
+     xw->visual    = DefaultVisualOfScreen(xw->screenptr);
 
-     attr.event_mask = 
+     attr.event_mask =
             ButtonPressMask
           | ButtonReleaseMask
           | PointerMotionMask
@@ -145,14 +148,19 @@ dfb_x11_open_window( DFBX11 *x11, XWindow** ppXW, int iXPos, int iYPos, int iWid
      xw->gc = XCreateGC(xw->display, xw->window, 0, NULL);
 
      // Create a null cursor
+     Pixmap  pixmp1;
+     Pixmap  pixmp2;
      XColor  fore;
      XColor  back;
      char    zero = 0;
 
-     xw->pixmp1     = XCreateBitmapFromData( xw->display, xw->window, &zero, 1, 1 );
-     xw->pixmp2     = XCreateBitmapFromData( xw->display, xw->window, &zero, 1, 1 );
+     pixmp1 = XCreateBitmapFromData( xw->display, xw->window, &zero, 1, 1 );
+     pixmp2 = XCreateBitmapFromData( xw->display, xw->window, &zero, 1, 1 );
 
-     xw->NullCursor = XCreatePixmapCursor( xw->display, xw->pixmp1, xw->pixmp2, &fore, &back, 0, 0 );
+     xw->NullCursor = XCreatePixmapCursor( xw->display, pixmp1, pixmp2, &fore, &back, 0, 0 );
+
+     XFreePixmap ( xw->display, pixmp1 );
+     XFreePixmap ( xw->display, pixmp2 );
 
      XDefineCursor( xw->display, xw->window, xw->NullCursor );
 
@@ -162,7 +170,7 @@ dfb_x11_open_window( DFBX11 *x11, XWindow** ppXW, int iXPos, int iYPos, int iWid
 
 
      if (x11->use_shm) {
-          // Shared memory 	
+          // Shared memory
           xw->shmseginfo=(XShmSegmentInfo *)D_CALLOC(1, sizeof(XShmSegmentInfo));
           if (!xw->shmseginfo) {
                x11->use_shm = false;
@@ -184,7 +192,7 @@ dfb_x11_open_window( DFBX11 *x11, XWindow** ppXW, int iXPos, int iYPos, int iWid
 
           /* we firstly create our shared memory segment with the size we need, and
           correct permissions for the owner, the group and the world --> 0777 */
-          xw->shmseginfo->shmid=shmget(IPC_PRIVATE, 
+          xw->shmseginfo->shmid=shmget(IPC_PRIVATE,
                                        xw->ximage->bytes_per_line * xw->ximage->height * 2,
                                        IPC_CREAT|0777);
 
@@ -240,7 +248,10 @@ no_shm:
 
           pitch = (xw->bpp * xw->width + 3) & ~3;
 
-          xw->virtualscreen = D_MALLOC( 2 * xw->height * pitch );
+          /* Use malloc(), not D_MALLOC() here, because XCreateImage()
+           * will call free() on this data.
+           */
+          xw->virtualscreen = malloc ( 2 * xw->height * pitch );
 
           xw->ximage = XCreateImage( xw->display, xw->visual, xw->depth, ZPixmap, 0,
                                      xw->virtualscreen, xw->width, xw->height * 2, 32, pitch );
@@ -272,19 +283,18 @@ void
 dfb_x11_close_window( DFBX11 *x11, XWindow* xw )
 {
      if (x11->use_shm) {
-          XShmDetach(xw->display, xw->shmseginfo);
-          shmdt(xw->shmseginfo->shmaddr);
-          shmctl(xw->shmseginfo->shmid,IPC_RMID,NULL);
-          D_FREE(xw->shmseginfo);
+          XShmDetach( xw->display, xw->shmseginfo );
+          shmdt( xw->shmseginfo->shmaddr );
+          shmctl( xw->shmseginfo->shmid, IPC_RMID, NULL );
+          D_FREE( xw->shmseginfo );
      }
-     else
-          D_FREE( xw->virtualscreen );
 
-     XDestroyImage(xw->ximage);
+     XDestroyImage( xw->ximage );
 
-     XFreeGC(xw->display,xw->gc);
-     XDestroyWindow(xw->display,xw->window);
-	
-     D_FREE(xw);
+     XFreeGC( xw->display, xw->gc );
+     XDestroyWindow( xw->display, xw->window );
+     XFreeCursor( xw->display, xw->NullCursor );
+
+     D_FREE( xw );
 }
 
