@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -49,6 +49,9 @@
 #include <fusion/reactor.h>
 #include <fusion/ref.h>
 
+#ifndef HAVE_FORK
+# define fork() -1
+#endif
 
 #define MSG(x...)                       \
 do {                                    \
