@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -55,7 +55,7 @@ static DFBColor
 blit_pixel( CardState *state, DFBColor src, DFBColor dst )
 {
      /* Scratch for blending stage. */
-     DFBColor x;
+     DFBColor x, dst_org;
 
      /*
       * Input => short circuit to Output? (simple blits)
@@ -82,6 +82,7 @@ blit_pixel( CardState *state, DFBColor src, DFBColor dst )
      /*
       * Modulation stage
       */
+     dst_org = dst;
 
      /* Modulate source alpha value with global alpha factor? */
      if (state->blittingflags & DSBLIT_BLEND_COLORALPHA) {
@@ -125,16 +126,6 @@ blit_pixel( CardState *state, DFBColor src, DFBColor dst )
           MODULATE( dst.b, dst.a );
      }
 
-     /*
-      * XOR comes right before blending, after load, modulate and premultiply.
-      */
-     if (state->blittingflags & DSBLIT_XOR) {
-          src.a ^= dst.a;
-          src.r ^= dst.r;
-          src.g ^= dst.g;
-          src.b ^= dst.b;
-     }
-
      /*
       * Blending stage
       */
@@ -329,6 +320,16 @@ blit_pixel( CardState *state, DFBColor src, DFBColor dst )
           x.b = ((int)x.b << 8) / ((int)x.a + 1);
      }
 
+     /*
+      * XOR comes at the end.
+      */
+     if (state->blittingflags & DSBLIT_XOR) {
+          x.a ^= dst_org.a;
+          x.r ^= dst_org.r;
+          x.g ^= dst_org.g;
+          x.b ^= dst_org.b;
+     }
+
      /*
       * Output
       */
