@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -48,7 +48,9 @@ static const DirectFBScreenMixerTreeNames( tree_names );
 static const DirectFBScreenEncoderScanModeNames( scan_mode_names );
 static const DirectFBScreenEncoderTestPictureNames( test_picture_names );
 static const DirectFBScreenEncoderTVStandardsNames( tv_standard_names );
+static const DirectFBScreenEncoderFrequencyNames( frequency_names );
 static const DirectFBScreenOutputSignalsNames( signal_names );
+static const DirectFBScreenOutputResolutionNames( resolution_names );
 
 /**************************************************************************************************/
 
@@ -290,10 +292,18 @@ static const AnyOption options[] = {
        &encoder_config.out_signals, &encoder_config.flags,
        DSECONF_OUT_SIGNALS, parse_enum, signal_names },
 
+     { "-er", "--resolution",   "<resolution>", "Set resolution",
+       &encoder_config.resolution, &encoder_config.flags,
+       DSECONF_RESOLUTION, parse_enum, resolution_names },
+
      { "-ec", "--scan-mode",    "<mode>",     "Set scan mode",
        &encoder_config.scanmode, &encoder_config.flags,
        DSECONF_SCANMODE, parse_enum, scan_mode_names },
 
+     { "-ef", "--frequency",    "<frequency>", "Set frequency",
+       &encoder_config.frequency, &encoder_config.flags,
+       DSECONF_FREQUENCY, parse_enum, frequency_names },
+
 
      { "-o",  "--output",       "<index>",    "Index of output to use",
        &output,  NULL, 0, parse_int, NULL }
@@ -604,6 +614,19 @@ test_picture_name( DFBScreenEncoderTestPicture test_picture )
      return "<invalid>";
 }
 
+static const char *
+resolution_name( DFBScreenOutputResolution resolution )
+{
+     int i;
+
+     for (i=0; i<D_ARRAY_SIZE(resolution_names); i++) {
+          if (resolution_names[i].resolution == resolution)
+               return resolution_names[i].name;
+     }
+
+     return "<invalid>";
+}
+
 static const char *
 scan_mode_name( DFBScreenEncoderTestPicture scan_mode )
 {
@@ -617,6 +640,19 @@ scan_mode_name( DFBScreenEncoderTestPicture scan_mode )
      return "<invalid>";
 }
 
+static const char *
+frequency_name( DFBScreenEncoderFrequency frequency )
+{
+     int i;
+
+     for (i=0; i<D_ARRAY_SIZE(frequency_names); i++) {
+          if (frequency_names[i].frequency == frequency)
+               return frequency_names[i].name;
+     }
+
+     return "<invalid>";
+}
+
 static void
 dump_encoder_config( const DFBScreenEncoderConfig *config )
 {
@@ -652,9 +688,15 @@ dump_encoder_config( const DFBScreenEncoderConfig *config )
           printf( "\n" );
      }
 
+     if (config->flags & DSECONF_RESOLUTION)
+          printf( "Resolution:    %s\n", resolution_name( config->resolution ) );
+
      if (config->flags & DSECONF_SCANMODE)
           printf( "Scan Mode:     %s\n", scan_mode_name( config->scanmode ) );
 
+     if (config->flags & DSECONF_FREQUENCY)
+          printf( "Frequency:     %s\n", frequency_name( config->frequency ) );
+
      printf( "\n" );
 }
 
