@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -220,10 +220,18 @@ load_image (const char            *filename,
                     for (s = data, d = dest; h; h--, s += pitch, d += d_pitch)
                          dfb_argb_to_rgb16 ((u32 *) s, (u16 *) d, width);
                     break;
+               case DSPF_ARGB8565:
+                    for (s = data, d = dest; h; h--, s += pitch, d += d_pitch)
+                         dfb_argb_to_argb8565 ((u32 *) s, (u8 *) d, width);
+                    break;
                case DSPF_ARGB1555:
                     for (s = data, d = dest; h; h--, s += pitch, d += d_pitch)
                          dfb_argb_to_argb1555 ((u32 *) s, (u16 *) d, width);
                     break;
+               case DSPF_RGBA5551:
+                    for (s = data, d = dest; h; h--, s += pitch, d += d_pitch)
+                         dfb_argb_to_rgba5551 ((u32 *) s, (u16 *) d, width);
+                    break;
                case DSPF_ARGB2554:
                     for (s = data, d = dest; h; h--, s += pitch, d += d_pitch)
                          dfb_argb_to_argb2554 ((u32 *) s, (u16 *) d, width);
