@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -260,6 +260,20 @@ write_glyph( DGIFFGlyphInfo *glyph, FT_GlyphSlot slot, void *dst, int pitch )
                               for (i=0; i<glyph->width; i++)
                                    dst32[i] = ((src[i] ^ 0xFF) << 24) | 0xFFFFFF;
                               break;
+                         case DSPF_ARGB8565:
+                              for (i=0, j = -1; i<glyph->width; ++i) {
+                                   u32 d = (src[i] << 16) | 0xFFFF;
+#ifdef WORDS_BIGENDIAN
+                                   dst8[++j] = (d >> 16) & 0xff;
+                                   dst8[++j] = (d >>  8) & 0xff;
+                                   dst8[++j] = (d >>  0) & 0xff;
+#else
+                                   dst8[++j] = (d >>  0) & 0xff;
+                                   dst8[++j] = (d >>  8) & 0xff;
+                                   dst8[++j] = (d >> 16) & 0xff;
+#endif
+                              }
+                              break;
                          case DSPF_ARGB4444:
                               for (i=0; i<glyph->width; i++)
                                    dst16[i] = (src[i] << 8) | 0xFFF;
@@ -272,6 +286,10 @@ write_glyph( DGIFFGlyphInfo *glyph, FT_GlyphSlot slot, void *dst, int pitch )
                               for (i=0; i<glyph->width; i++)
                                    dst16[i] = (src[i] << 8) | 0x7FFF;
                               break;
+                         case DSPF_RGBA5551:
+                              for (i=0; i<glyph->width; i++)
+                                   dst16[i] = 0xFFFE | ((src[i] & 0x80) >> 7);
+                              break;
                          case DSPF_A8:
                               direct_memcpy( dst, src, glyph->width );
                               break;
@@ -306,6 +324,21 @@ write_glyph( DGIFFGlyphInfo *glyph, FT_GlyphSlot slot, void *dst, int pitch )
                                    dst32[i] = (((src[i>>3] & (1<<(7-(i%8)))) ?
                                                 0x00 : 0xFF) << 24) | 0xFFFFFF;
                               break;
+                         case DSPF_ARGB8565:
+                              for (i=0, j = -1; i<glyph->width; ++i) {
+                                   u32 d = (((src[i>>3] & (1<<(7-(i%8)))) ?
+                                             0xFF : 0x00) << 16) | 0xFFFF;
+#ifdef WORDS_BIGENDIAN
+                                   dst8[++j] = (d >> 16) & 0xff;
+                                   dst8[++j] = (d >>  8) & 0xff;
+                                   dst8[++j] = (d >>  0) & 0xff;
+#else
+                                   dst8[++j] = (d >>  0) & 0xff;
+                                   dst8[++j] = (d >>  8) & 0xff;
+                                   dst8[++j] = (d >> 16) & 0xff;
+#endif
+                              }
+                              break;
                          case DSPF_ARGB4444:
                               for (i=0; i<glyph->width; i++)
                                    dst16[i] = (((src[i>>3] & (1<<(7-(i%8)))) ?
@@ -321,6 +354,11 @@ write_glyph( DGIFFGlyphInfo *glyph, FT_GlyphSlot slot, void *dst, int pitch )
                                    dst16[i] = (((src[i>>3] & (1<<(7-(i%8)))) ?
                                                 0x1 : 0x0) << 15) | 0x7FFF;
                               break;
+                         case DSPF_RGBA5551:
+                              for (i=0; i<glyph->width; i++)
+                                   dst16[i] = 0xFFFE | ((src[i>>3] & (1<<(7-(i%8)))) ?
+                                                0x1 : 0x0);
+                              break;
                          case DSPF_A8:
                               for (i=0; i<glyph->width; i++)
                                    dst8[i] = (src[i>>3] &
