@@ -1,5 +1,5 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
@@ -128,7 +128,8 @@ context_notify( WMData                          *data,
      if (notification->flags & UCNF_DESTROYED) {
           D_DEBUG_AT( WM_Unique, "  -> context destroyed.\n" );
 
-          stack_data->context = NULL;
+          if (notification->context == stack_data->context)
+               stack_data->context = NULL;
 
           return RS_REMOVE;
      }
@@ -385,6 +386,9 @@ wm_set_active( CoreWindowStack *stack,
 
      D_MAGIC_ASSERT( data, StackData );
 
+     D_DEBUG_AT( WM_Unique, "%s( stack %p, wm_data %p, stack_data %p, %sactive )\n",
+                 __FUNCTION__, stack, wm_data, stack_data, active ? "" : "in" );
+
      if (!data->context) {
           D_ASSERT( !active );
           return DFB_OK;
@@ -830,6 +834,19 @@ wm_request_focus( CoreWindow *window,
      return unique_window_request_focus( data->window );
 }
 
+static DFBResult
+wm_begin_updates( CoreWindow      *window,
+                  void            *wm_data,
+                  void            *window_data,
+                  const DFBRegion *update )
+{
+     D_ASSERT( window != NULL );
+     D_ASSERT( wm_data != NULL );
+     D_ASSERT( window_data != NULL );
+
+     return DFB_OK;
+}
+
 /**************************************************************************************************/
 
 static DFBResult
